<?php

include_once("/usr/local/www/berogui/misc/database.php");
include_once '/usr/fallback/beroConf.php';

/**
 * Ehemalige ini_to_db Datei. Ermöglicht das wiederherstellen von Konfigurationsdateien. Die Konfigurationsdateien werden geparst 
 * und die Daten in der Datenkank gespeichert
 * @author Miguel Palmer <mp@beronet.com>
 */
class restoreConfig {

	private $_ba;
	private $_bc;
	private $_log;
	private $_files;
	private $_strpath;
	/**
	 * @var int Wenn der Wert 0 ist, sind manche Dateien nicht erforderlich damit das Restore erfolgreich ist
	 */
	private $_value;

	function __construct($value = 0) {
		$this->_ba = new beroAri();
		$this->_files = array('isgw.conf', 'hardware.conf', 'isgw.isdn', 'isgw.dialplan',
			'isgw.sip', 'misc.conf', 'isgw.causes', 'isgw.analog', 'isgw.cas',
			'isgw.cpt', 'network.acl', 'isgw.gsm', 'isgw.lte', 'isgw.whitelist');
		$this->_strpath = "/usr/conf/";
		$this->_value = $value;
		$this->_bc = new beroConf('root');
	}

	private function error($str, $line) {
		$this->log("FAILURE:" . $str . " in line:" . $line . "\n");
		return false;
	}

	private function log($line) {
		$this->_log .= $line . "\n";
	}

	public function getLog() {
		return $this->_log;
	}

	private function _parseFile($fileName) {
		$this->log("Parsing " . $fileName);
		$fn = explode('.', $fileName);
		$fn = implode("", $fn);
		$result = $this->$fn();
		return $result;
	}

	public function restoreConfig() {
		foreach ($this->_files as $file) {
			$result = $this->_parseFile($file);
			if (!$result)
				return false;
		}
		return true;
	}

	private function _makeParseableInifile($sFile, $dFile) {

		$handle = @fopen($sFile, "r"); // Open file form read.
		$whandle = fopen($dFile, "w");
		if ($handle) {
			while (!feof($handle)) { // Loop til end of file.
				$buffer = fgets($handle, 4096); // Read a line.
				$eqpos = strpos($buffer, "=");
				//Alle Zeilen maskieren da manche Zeilen in dem Value ein = Zeichen enthalten
				if ($eqpos !== false) {
					$key = trim(substr($buffer, 0, $eqpos));
					$value = trim(substr($buffer, $eqpos + 1));
					$buffer = $key . "=\"" . $value . "\"\n";
				}

				if (strpos($key, "tones") !== false) {
					$buffer = str_replace("[", "", $buffer);
					$buffer = str_replace("]", "", $buffer);
				}
				if (strpos($key, "overlap_dialing") !== false) {
					$buffer = str_replace("yes", "1", $buffer);
					$buffer = str_replace("no", "0", $buffer);
				}
				if (strpos($key, "sip_over_tcp") !== false) {
					$buffer = str_replace("yes", "1", $buffer);
					$buffer = str_replace("no", "0", $buffer);
				}
				if (strpos($key, "time_from_isdn") !== false) {
					$buffer = str_replace("yes", "1", $buffer);
					$buffer = str_replace("no", "0", $buffer);
				}
				if (strpos($key, "qsig") !== false) {
					$buffer = str_replace("yes", "1", $buffer);
					$buffer = str_replace("no", "0", $buffer);
				}
				if (strpos($buffer, "lan-dhcp") !== false) {
					$buffer = str_replace("yes", "1", $buffer);
					$buffer = str_replace("no", "0", $buffer);
				}
				if (strpos($key, "use-boot-prov") !== false) {
					$buffer = str_replace("yes", "1", $buffer);
					$buffer = str_replace("no", "0", $buffer);
				}
				if (strpos($key, "bnfos-hb-on-boot") !== false) {
					$buffer = str_replace("yes", "1", $buffer);
					$buffer = str_replace("no", "0", $buffer);
				}

				$wbuffer = fwrite($whandle, $buffer);
			}
			fclose($handle); // Close the file.
		}
		return;
	}

	private function _saveKeyVal($key, $value, $default = '') {
		if ($value == '') {
			return $this->_ba->set($key, sqlite_escape_string($default));
		} else {
			return $this->_ba->set($key, sqlite_escape_string($value));
		}
	}

	private function _rootDb($key, $value, $check = 0) {
		if ($check == 1 && $value != "") {
			$check = 0;
		}
		if ($check == 0) {
			$this->_bc->set("root", $key, sqlite_escape_string($value));
		}
	}

	private function _insert($table, $keys, $values) {
		//Build query
		$structure = 'INSERT INTO ' . $table . ' (';
		foreach ($keys as $key) {
			$structure .= $key . ',';
		}
		$structure = trim($structure, ",");
		$structure .= ') VALUES (';
		foreach ($values as $value) {
			$structure .= '"' . sqlite_escape_string($value) . '",';
		}
		$structure = trim($structure, ",");
		$structure .= ');';
		$this->log($structure);
		return $this->_ba->insert_($structure);
	}

	private function _update($table, $values, $where) {
		//Build query
		$structure = 'UPDATE ' . $table . ' SET ';
		foreach ($values as $key => $value) {
			$structure .= $key . '="' . sqlite_escape_string($value) . '",';
		}
		$structure = trim($structure, ",");
		$structure .= ' ' . $where . ';';
		$this->log($structure);
		return $this->_ba->update($structure);
	}

	public function isgwconf() {
		if (!file_exists($this->_strpath . "isgw.conf")) {
			return $this->error("no isgw.conf found", __LINE__);
		}

		$result = $this->_makeParseableInifile($this->_strpath . "isgw.conf", $this->_strpath . "isgw.conf.tmp");

		$result = parse_ini_file($this->_strpath . "isgw.conf.tmp", true);
		//Array $key=>$default
		$fields = array(
			'logserver' => '127.0.0.1:54323',
			'logging_active' => '0',
			'logging_message' => '1',
			'logging_debug' => '0',
			'loglevel_debug' => '0',
			'syslog_facility' => '16',
			'experimental_options' => '',
			'pcm_server' => '0',
			'pcm_server_port' => '54329',
			'logcdr' => '0',
			'reject_calls_under_load' => '0',
			'emergency_numbers' => '110|112|911|999',
			'blacklist_max_packets' => '0',
			'blacklist_timeframe' => '0',
			'blacklist_blockduration' => '86400',
		);

		foreach ($result as $section => $values) {

			if ($section == "general") {


				$loglevel = $result[$section][loglevel];
				$loglevel_message = $result[$section][loglevel_message];
				//Migration step 3.0
				if ($loglevel_message == "") {
					$loglevel_message = $loglevel;
				}
				if ($loglevel_message == "") {
					$loglevel_message = "2";
				}
				$this->_ba->set('loglevel_message', sqlite_escape_string($loglevel_message));

				foreach ($fields as $key => $default) {
					$res = $this->_saveKeyVal($key, $result[$section][$key], $default);
					if (!$res)
						return error("isgwconf insert failed", __LINE__);
				}
			}
		}
		return true;
	}

	function isgwdialplan() {
	
		if (!file_exists($this->_strpath . "isgw.dialplan")) {
			return $this->error("isgw.dialplan not found", __LINE__);
		}
		$handle = @file_get_contents($this->_strpath . "isgw.dialplan"); // Open file.
		$res = $this->_ba->delete("delete from dialplan");
		if ($handle) {
			$posid = 0;
			$lines = explode("\n", $handle);
			foreach ($lines as $buffer) { // Loop til end of file.
				$ss = 0;
				if ($buffer == ";From\tTo\tFromID\tToID\tDestination\tNewDestination\tSource\tNewSource\tConfig" || strlen($buffer) == 0) {
					continue;
				}
				$result = explode("\t", $buffer);
				if (count($result) > "1") {
					$isdnid = "";
					if (substr($result[4], 0, 10) == "ISDNZUISDN") {
						$isdnid = substr($result[4], 0, 23);
						$result[4] = substr($result[4], 23);
						$ss = 1;
					}
					if (substr($result[5], 0, 10) == "ISDNZUISDN") {
						$isdnid = substr($result[5], 0, 23);
						$result[5] = substr($result[5], 23);
						$ss = 0;
					}

					if (strpos($result[0], ";#") === FALSE) {
						$active = 1;
					} else {
						$result[0] = str_replace(";#", "", $result[0]);
						$active = 0;
					}

					if ($ss == 0) {
						$posid = $posid + 1;
						$keys = array('fromDirection', 'toDirection', 'fromId',
							'toId', 'dest', 'newDest', 'src', 'newSrc', 'config',
							'isdnid', 'posid', 'activ');
						$values = $result;
						if (count($values) == 8) {
							$values[] = "";
						}
						$values[] = $isdnid;
						$values[] = $posid;
						$values[] = $active;
						$values = str_replace("\\", "\\\\", $values);
						$values = str_replace('""', '\"\"', $values);
						$res = $this->_insert('dialplan', $keys, $values);
					} else {
						$strsql = "update dialplan set 'toID'= '" . sqlite_escape_string($result[3]) . "', toDirection='" . sqlite_escape_string($result[1]) . "' where isdnid='" . sqlite_escape_string($isdnid) . "'";
						$this->log($strsql);
						$res = $this->_ba->insert_($strsql);
					}

					if (!$res)
						return $this->error("isgwdialplan insert failed", __LINE__);
				}
			}
		}
		return true;
	}
	
	function isgwwhitelist() {

		if (!file_exists($this->_strpath . "isgw.whitelist")) {
			if ($this->_value == 0) {
				return $this->error("no isgw.whitelist", __LINE__);
			}
			else {
				return true;
			}
		}
		$handle = @file_get_contents($this->_strpath . "isgw.whitelist"); // Read file.
		
		if ($handle) {
			$i = 0;
			$lines = explode("\n", $handle);
			foreach ($lines as $buffer) {
				$fields = explode("/", $buffer);
				if (count ($fields) == 2) {
					if ($fields[1] < 0 || $fields[1] > 32) {
						continue;
					}
					if ($fields[0] == "") {
						continue;
					}
					$strsql = "INSERT INTO whitelist (address, netmask) VALUES ('" . sqlite_escape_string($fields[0]) . "','" . sqlite_escape_string($fields[1]) . "')";
				}
				else {
					continue;
				}
				$res = $this->_ba->insert_($strsql);
			}
		}
		return true;
	}
	
	function isgwcpt() {
		if (!file_exists($this->_strpath . "isgw.cpt")) {
			if ($this->_value == 0) {
				return $this->error("no isgw.cpt", __LINE__);
			} else {
				return true;
			}
		}
		$handle = @file_get_contents($this->_strpath . "isgw.cpt"); // Read file.
		$res = $this->_ba->delete("delete from callprogress");
		$res = $this->_ba->delete("delete from callprogress_entries");
		if ($handle) {
			$i = 0;
			$lines = explode("\n", $handle);
			foreach ($lines as $buffer) {
				$result = explode("\t", $buffer);

				if (substr($result[0], 0, 1) == "[") {
					$i++;
					$section = substr($result[0], 1, strlen($result[0]) - 2);
					$res = explode(":", $section);

					if ($res[0] == "s2i") {
						$cptDir = 'SIP-ISDN';
					} elseif ($res[0] == "i2s") {
						$cptDir = 'ISDN-SIP';
					} elseif ($res[0] == "s2fxo") {
						$cptDir = 'SIP-FXO';
					}
					$keys = array('id', 'direction', 'name');
					$values = array($i, $cptDir, $res[1]);
					$res = $this->_insert('callprogress', $keys, $values);
					if (!$res) {
						return $this->error("isgw.cpt: tables name insert failed", __LINE__);
					}
				} elseif (substr($result[0], 0, 5) == "TIMER") {
					$res = explode("=", $result[0]);
					$strsql = "UPDATE callprogress set " . sqlite_escape_string(strtolower($res[0])) . "='" . sqlite_escape_string($res[1]) . "' WHERE id='" . $i . "'";
					$res = $this->_ba->update($strsql);
				} elseif (count($result) > 2) {
					$res = null;
					switch ($cptDir) {
						case 'SIP-ISDN':
							$keys = array('type', 'chann_det', 'isdn_event', 'sip_resp', 'resp_text', 'id_name');
							$values = $result;
							$values[] = $i;
							$res = $this->_insert('callprogress_entries', $keys, $values);
							break;
						case 'ISDN-SIP':
							$keys = arraiy('type', 'isdn_event', 'resp_text', 'sip_resp', 'id_name','chann_det');
							$values = $result;
							$values[] = $i;
							$values[]='';
							$res = $this->_insert('callprogress_entries', $keys, $values);
							break;
						case 'SIP-FXO':
							$keys = array('isdn_event', 'sip_resp', 'resp_text', 'id_name','chann_det','type');
							$values = $result;
							$values[] = $i;
							$values[]='';
							$values[]='';
							$res = $this->_insert('callprogress_entries', $keys, $values);
							break;
					}
					if (!$res) {
						return $this->error("isgwcpt: table entries insert failed", __LINE__);
					}
				}
			}
		}
		return true;
	}

	public function isgwsip() {
		if (!file_exists($this->_strpath . "isgw.sip")) {
			return $this->error("no isgw.sip found", __LINE__);
		}

		$result = $this->_makeParseableInifile($this->_strpath . "isgw.sip", $this->_strpath . "isgw.sip.tmp");
		$result = parse_ini_file($this->_strpath . "isgw.sip.tmp", true);
		$res = $this->_ba->delete("delete from sip_general");
		$res = $this->_ba->delete("delete from sip");
		$i = 0;
		foreach ($result as $section => $values) {
			if ($section == "general") {
				$networtInterface = array('lan', 'wan', 'vpn');
				$rtpport=5000;
				foreach ($networtInterface as $interface) {
					if ($interface === 'lan' || isset($result[$section]['sip_bindport_' . $interface])) {
						$fields = array(
							'sip_bindport_' . $interface => '5060',
							'rtpports_' . $interface => ($rtpport+=1000) . '-' . ($rtpport+=1000) , 
							'tos_sip_' . $interface => '160',
							'tos_rtp_' . $interface => '160',
							'rtp_port_selection_' . $interface => '',
							'ssl_version_' . $interface => '',
							'sip_transport_' . $interface => '',
							'stack_active_' . $interface => '1',
						);
						foreach ($fields as $key => $default) {
							$this->_saveKeyVal($key, $result[$section][$key], $default);
						}
					}
				}
			} else {
				$i++;
				//Sever address was replaced by sip_registrar, sip_domain and sip_proxy
				$address = $result[$section]['address'];
				$sip_registrar = $result[$section]['sip_registrar'];
				$sip_domain = $result[$section]['sip_domain'];
				$sip_proxy = $result[$section]['sip_proxy'];
				//Migration from the old config file
				if ($sip_proxy == "" && $sip_registrar == "" && $sip_domain == "" && $address != "") {
					$result[$section]['sip_proxy'] = $address;
					$result[$section]['sip_registrar'] = $address;
					$result[$section]['sip_domain'] = $address;
				}

				$register = $result[$section]['register'];

				if ($register == "") {
					$register = $result[$section]['type'];
					if ($register == "") {
						$result[$section]['register'] = 0;
					} else {
						if ($register == "both") {
							$result[$section]['register'] = 1;
						} else {
							$result[$section]['register'] = 0;
						}
					}
				}
				$result[$section]['def'] = $result[$section]['default'];
				$result[$section]['sip_transport'] = $result[$section]['transport'];
				$result[$section]['register_option'] = $result[$section]['register_options'];
				$result[$section]['enum_search_domain'] = $result[$section]['enum_search_domains'];
				$result[$section]['keepalive_interval'] = $result[$section]['keepalive-interval'];
				$result[$section]['name'] = $section;
				$keys = array('name' => '', 'sip_proxy' => '', 'sip_domain' => '', 'sip_registrar' => '',
					'user' => '', 'auth_user' => '', 'displayname' => '', 'sip_transport' => '', 'secret' => '', 'register' => '', 'config' => '', 'def' => '0', 'stunserver' => '',
					'externip' => '', 'register_option' => '', 'regrefresh' => "300", 'matchtype' => '', 'matchtype_manual' => '', 'profile' => 'manual', 'enum_support' => '0', 'enum_search_domain' => '',
					'keepalive_interval' => '0', 'call_transfer_method' => 'refer', 'sip_custom_port' => '', 'sip_stack' => 'lan');
				$values = array();
				foreach ($keys as $key => $default) {
					if ($result[$section][$key] == '') {
						$values[] = $default;
					} else {
						$values[] = $result[$section][$key];
					}
				}
				$res = $this->_insert('sip', array_keys($keys), $values);

				if (!$res) {
					return $this->error("isgwsip: table sip insert failed", __LINE__);
				}
			}
		}
		$res = unlink($this->_strpath . "isgw.sip.tmp");
		return true;
	}

	public function isgwisdn() {

		if (!file_exists($this->_strpath . "isgw.isdn")) {
			return $this->error("no isgw.isdn found", __LINE__);
		}

		$result = $this->_makeParseableInifile($this->_strpath . "isgw.isdn", $this->_strpath . "isgw.isdn.tmp");
		$result = parse_ini_file($this->_strpath . "isgw.isdn.tmp", true);

		foreach ($result as $section => $values) {
			if ($section != "general") {
				$ports = $result[$section]['ports'];
				//Array $colname|$default
				$keys = array('chansel', 'chansel_dir', 'config', 'tones', 'portpullup', 'odt|3',
					'pcmlaw|default', 'country_code', 'city_code', 'local_area_code', 'dialtone_digits',
					'overlap_dialing|0', 'qsig|0', 'odt_empty_dad|15', 'restart_after_link_loss',
					'gr|' . $section);
				$result[$section]['chansel_dir'] = $result[$section]['chansel_direction'];
				$result[$section]['tones'] = '[' . $result[$section]['tones'] . ']';
				$result[$section]['odt'] = $result[$section]['interdigit_timeout'];
				$result[$section]['odt_empty_dad'] = $result[$section]['interdigit_timeout_initial'];

				$port = explode(",", $ports);

				foreach ($port as $a) {
					$values = array();
					foreach ($keys as $key) {
						$key = explode('|', $key);
						$values[$key[0]] = ($result[$section][$key[0]] == "") ? $key[1] : $result[$section][$key[0]];
					}
					$where = " where port='" . sqlite_escape_string($a) . "'";
					$res = $this->_update('isdn', $values, $where);

					if (!$res)
						return $this->error("isgwisdn: table isdn update failed", __LINE__);
				}
			}else {
				$time_from_isdn = $result[$section][time_from_isdn];
				if ($time_from_isdn == "")
					$time_from_isdn = "0";
				$this->_rootDb("time_from_isdn", $time_from_isdn);

				if ($time_from_isdn != "0")
					$this->_rootDb("ntphost", '');
			}
		}
		@unlink($this->strpath . "isgw.isdn.tmp");
		return true;
	}

	public function isgwanalog() {

		if (!file_exists($this->_strpath . "isgw.analog")) {
			return $this->error("no isgw.analog found", __LINE__);
		}

		$result = $this->_makeParseableInifile($this->_strpath . "isgw.analog", $this->_strpath . "isgw.analog.tmp");
		$result = parse_ini_file($this->_strpath . "isgw.analog.tmp", true);

		foreach ($result as $section => $values) {
			if ($section != "general") {
				$ports = $result[$section]['ports'];
				$result[$section]['overlap_dial_timeout_empty_dad'] = $result[$section]['interdigit_timeout_initial'];
				$result[$section]['overlap_dial_timeout'] = $result[$section]['interdigit_timeout'];
				$result[$section]['tones'] = '[' . $result[$section]['tones'] . ']';
				$result[$section]['chansel_dir'] = $result[$section]['chansel_direction'];
				$result[$section]['gr'] = $section;
				$port = explode(",", $ports);
				$keys = array('config', 'overlap_dial_timeout_empty_dad', 'overlap_dial_timeout',
					'overlap_dialing', 'fxo_connect', 'fxo_wait_for_oad', 'message_waiting_method', 'tones',
					'clip', 'cnip', 'chansel', 'chansel_dir', 'fxo_dialtone_passthrough', 'fxo_call_ending_signal',
					'fxo_cid_detection_mode', 'dialtone_digits', 'gr');

				foreach ($port as $a) {
					$values = array();
					foreach ($keys as $key) {
						$values[$key] = $result[$section][$key];
					}
					$where = " where port='" . sqlite_escape_string($a) . "'";
					$res = $this->_update('analog', $values, $where);

					if (!$res)
						return $this->error("isgwanalog: table analog update failed", __LINE__);
				}
			}
		}
		@unlink($this->_strpath . "isgw.analog.tmp");
		return true;
	}

	public function isgwcas() {

		if (!file_exists($this->_strpath . "isgw.cas")) {
			if ($this->_value == 0) {
				return $this->error("no isgw.cas", __LINE__);
			} else {
				return true;
			}
		}

		$result = $this->_makeParseableInifile($this->_strpath . "isgw.cas", $this->_strpath . "isgw.cas.tmp");
		$result = parse_ini_file($this->_strpath . "isgw.cas.tmp", true);

		foreach ($result as $section => $values) {
			if ($section != "general") {
				$ports = $result[$section][ports];
				$result[$section]['e1sigtype'] = $result[$section]['e1_signalling'];
				$result[$section]['t1sigtype'] = $result[$section]['t1_signalling'];
				$result[$section]['tones'] = '[' . $result[$section]['tones'] . ']';
				$result[$section]['chansel_dir'] = $result[$section]['chansel_direction'];
				$result[$section]['gr'] = $section;
				$port = explode(",", $ports);
				$keys = array('config', 'interdigit_timeout_initial', 'interdigit_timeout',
					'overlap_dialing', 'fxo_wait_for_oad', 'tones', 'fxo_callerid_detection_timeout',
					'clip', 'cnip', 'chansel', 'chansel_dir', 'fxo_call_ending_signal', 'pcmlaw',
					'fxo_cid_detection_mode', 'dialtone_digits', 'gr', 'e1sigtype', 't1sigtype', 'wait_for_oad');

				foreach ($port as $a) {
					$values = array();
					foreach ($keys as $key) {
						$values[$key] = $result[$section][$key];
					}
					$where = " where channel='" . sqlite_escape_string($a) . "'";
					$res = $this->_update('cas_channels', $values, $where);

					if (!$res)
						return $this->error("isgwcas: table cas_channels update failed", __LINE__);
				}
			}
		}
		@unlink($this->_strpath . "isgw.cas.tmp");
		return true;
	}

	public function isgwgsm() {

		if (!file_exists($this->_strpath . "isgw.gsm"))
			return $this->error("no isgw.gsm found", __LINE__);

		$result = $this->_makeParseableInifile($this->_strpath . "isgw.gsm", $this->_strpath . "isgw.gsm.tmp");
		$result = parse_ini_file($this->_strpath . "isgw.gsm.tmp", true);

		foreach ($result as $section => $values) {
			if ($section != "general") {
				$ports = $result[$section][ports];
				$result[$section]['sms_extension'] = $result[$section]['sms_extensions'];
				$result[$section]['extension'] = $result[$section]['extensions'];
				$result[$section]['chansel_dir'] = $result[$section]['chansel_direction'];
				$result[$section]['gr'] = $section;
				$port = explode(",", $ports);
				$keys = array('config', 'chansel', 'chansel_dir', 'sms_extension',
					'extension', 'gr');

				$port = explode(",", $ports);
				foreach ($port as $a) {
					$values = array();
					foreach ($keys as $key) {
						$values[$key] = $result[$section][$key];
					}
					$where = " where port='" . sqlite_escape_string($a) . "'";
					$res = $this->_update('gsm', $values, $where);

					if (!$res)
						return $this->error("isgwgsm: table gsm update failed", __LINE__);
				}
			}else {
				$ports = $this->_ba->select("select port from gsm");
				$codec = explode(",", $result[$section]['codec']);
				$pin = explode(",", $result[$section]['pin']);
				$smsc = explode(",", $result[$section]['smsc']);
				while ($entry = $this->_ba->fetch_array($ports)) {
					$key = $entry['port'] - 1;
					$values = array('pin' => $pin[$key], 'smsc' => $smsc[$key],
						'codec' => $codec[$key]);
					$where = " where port='" . $entry['port'] . "'";
					$res = $this->_update('gsm', $values, $where);
				}
			}
		}
		@unlink($strpath . "isgw.gsm.tmp");
		return true;
	}
	
	public function isgwlte() {

		if (!file_exists($this->_strpath . "isgw.lte")) {
			if ($this->_value == 0) {
				return $this->error("no isgw.lte found", __LINE__);
			} else {
				return true;
			}
		}

		$result = $this->_makeParseableInifile($this->_strpath . "isgw.lte", $this->_strpath . "isgw.lte.tmp");
		$result = parse_ini_file($this->_strpath . "isgw.lte.tmp", true);

		foreach ($result as $section => $values) {
			if ($section != "general") {
				$ports = $result[$section][ports];
				$result[$section]['sms_extension'] = $result[$section]['sms_extensions'];
				$result[$section]['extension'] = $result[$section]['extensions'];
				$result[$section]['chansel_dir'] = $result[$section]['chansel_direction'];
				$result[$section]['gr'] = $section;
				$port = explode(",", $ports);
				$keys = array('config', 'chansel', 'chansel_dir', 'sms_extension',
					'extension', 'gr');

				$port = explode(",", $ports);
				foreach ($port as $a) {
					$values = array();
					foreach ($keys as $key) {
						$values[$key] = $result[$section][$key];
					}
					$where = " where port='" . sqlite_escape_string($a) . "'";
					$res = $this->_update('lte', $values, $where);

					if (!$res)
						return $this->error("isgwlte: table lte update failed", __LINE__);
				}
			}else {
				$ports = $this->_ba->select("select port from lte");
				$codec = explode(",", $result[$section]['codec']);
				$pin = explode(",", $result[$section]['pin']);
				$smsc = explode(",", $result[$section]['smsc']);
				while ($entry = $this->_ba->fetch_array($ports)) {
					$key = $entry['port'] - 1;
					$values = array('pin' => $pin[$key], 'smsc' => $smsc[$key],
						'codec' => $codec[$key]);
					$where = " where port='" . $entry['port'] . "'";
					$res = $this->_update('lte', $values, $where);
				}
			}
		}
		@unlink($strpath . "isgw.lte.tmp");
		return true;

	}


	public function isgwcauses() {

		if (!file_exists($this->_strpath . "isgw.causes"))
			return $this->error("no isgw.causes", __LINE__);

		$causes = @file($this->strpath . "isgw.causes");
		if ($causes != NULL) {
			$this->_ba->delete("Delete from isgw_causes");
			foreach ($causes as $key => $line) {
				if (substr_count($line, ";", 0, 1) == 0 && $line != "" && $line != "\r" && $line != "\n" && $line != "\r\n") {
					$line = str_replace(array("\r\n", "\n", "\r"), "", $line);
					$values = explode("\t", $line);
					$keys = array('direction', 'to_cause', 'from_cause', 'sip_response_message');
					$res = $this->_insert('isgw_causes', $keys, $values);
					if (!$res)
						return $this->error("isgwcauses: table isgw_causes insert failed", __LINE__);
				}
			}
		}
		return true;
	}

	public function networkacl() {

		if (!file_exists($this->_strpath . "network.acl")) {
			if ($this->_value == 0)
				return$this->error("no network.acl", __LINE__);
			else
				return true;
		}

		$acl = @file($this->_strpath . "network.acl");
		if ($acl != NULL) {
			$this->_ba->delete("Delete from acl");

			foreach ($acl as $key => $line) {
				if ($line != "" && $line != "\r" && $line != "\n" && $line != "\r\n") {
					$line = str_replace(array("\r\n", "\n", "\r"), "", $line);
					$values = explode(";", $line);
					$address = explode("/", $values[3]);
					
					//ignore invalid lines with no port defined
					if ($values[2] == "") {
						continue;
					}
					$strsql = "INSERT INTO acl (protocol, dest_port, address, netmask)
					VALUES ('" . sqlite_escape_string($values[1]) . "','" . sqlite_escape_string($values[2]) . "','" . sqlite_escape_string($address[0]) . "', '" . sqlite_escape_string($address[1]) . "')";
					if ($DEBUG)
						$this->log($strsql . " \n");
					$res = $this->_ba->insert_($strsql);
				}
			}
		}
		return true;
	}

	function miscconf() {
		if (!file_exists($this->_strpath . "misc.conf"))
			return $this->error("no misc.conf", __LINE__);

		$result = $this->_makeParseableInifile($this->_strpath . "misc.conf", $this->_strpath . "misc.conf.tmp");
		$misc = parse_ini_file($this->_strpath . "misc.conf.tmp", true);

		foreach ($misc as $section => $values) {
			//Time Settings
			$this->_rootDb("ntphost", $misc[$section]['ntphost']);
			$this->_rootDb("timezone", $misc[$section]['timezone']);
			$this->_saveKeyVal("summer_winter_time", $misc[$section]['summer_winter_time'], '');

			//berofos settings
			$this->_saveKeyVal("ip_berofos", $misc[$section]['bnfos-ipaddr'], '');
			$this->_saveKeyVal("mac_berofos", $misc[$section]['bnfos-mac'], '');
			$this->_saveKeyVal("interval_berofos", $misc[$section]['bnfos-intv'], '');
			$this->_rootDb("use_boot_bnfoswdog", $misc[$section]['bnfos-hb-on-boot'], 1);

			//LAN Settings
			$this->_rootDb("lan-mode", $misc[$section]['lan-mode'], 1);
			$this->_rootDb("lan-iface", $misc[$section]['lan-iface'], 1);
			$this->_rootDb("lan-ipaddr", $misc[$section]['lan-ipaddr'], 1);
			$this->_rootDb("lan-netmask", $misc[$section]['lan-netmask'], 1);
			$this->_rootDb("lan-gateway", $misc[$section]['lan-gateway'], 1);
			$this->_rootDb("lan-nameserver", $misc[$section]['lan-nameserver'], 1);
			$this->_rootDb("lan-dhcp", $misc[$section]['lan-dhcp'], 1);
			$this->_rootDb("lan-mtu", $misc[$section]['lan-mtu'], 1);
			$this->_rootDb("routing", $misc[$section]['routing']);
			$this->_saveKeyVal("hostname", $misc[$section]['hostname'], '');

			//WAN Settings
			$this->_rootDb("wan-ipaddr", $misc[$section]['wan-ipaddr'], 1);
			$this->_rootDb("wan-netmask", $misc[$section]['wan-netmask'], 1);
			$this->_rootDb("wan-gateway", $misc[$section]['wan-gateway'], 1);
			$this->_rootDb("wan-dhcp", $misc[$section]['wan-dhcp'], 1);

			//Bonding Settings
			$this->_rootDb("bonding-mode", $misc[$section]['bonding-mode'], 1);
			$this->_rootDb("bonding-miimon", $misc[$section]['bonding-miimon'], 1);
			$this->_rootDb("bonding-updelay", $misc[$section]['bonding-updelay'], 1);
			$this->_rootDb("bonding-downdelay", $misc[$section]['bonding-downdelay'], 1);

			//VLAN settings
			$this->_rootDb("vlan-enable", $misc[$section]['vlan-enable']);
			$this->_rootDb("vlan-id", $misc[$section]['vlan-id']);
			$this->_rootDb("vlan-prio", $misc[$section]['vlan-prio']);

			//MSP Settings
			$this->_rootDb("msp-iface", $misc[$section]['msp-iface'], 1);
			$this->_rootDb("msp-ipaddr", $misc[$section]['msp-ipaddr'], 1);

			//Provisioning
			$this->_rootDb("use-boot-prov", $misc[$section]['use-boot-prov'], 1);
			$this->_rootDb("tftp-host", $misc[$section]['tftp-host']);
			$this->_rootDb("tftp-url", $misc[$section]['tftp-url']);
			$this->_rootDb("provisioning_manual", $misc[$section]['provisioning_manual'], 1);
			$this->_rootDb("provisioning_mode", $misc[$section]['provisioning_mode'], 1);
			$this->_rootDb("provisioning_url", $misc[$section]['provisioning_url'], 1);
			$this->_rootDb("polling_interval", $misc[$section]['polling_interval'], 1);

			$this->_rootDb("userapp-enabled", $misc[$section]['userapp-enabled']);
			$this->_rootDb("beroAvahi-enabled", $misc[$section]['beroAvahi-enabled']);

			//Logging
			$this->_rootDb("system-check-enabled", $misc[$section]['system-check-enabled']);

			//Security
			$this->_rootDb("safeguard", $misc[$section]['safeguard']);
			$this->_rootDb("excludelan", $misc[$section]['excludelan']);

			//user
			$adminpass = $misc[$section]['admin-pass'];
			$adminuser = $misc[$section]['admin-user'];
			if (($adminpass != "") || ($adminuser == "admin")) {
				$values = array('password' => $adminpass);
				$where = " where user='" . sqlite_escape_string($adminuser) . "'";
				$this->_update('users', $values, $where);
			}
			//misc
			$this->_saveKeyVal("national_code", $misc[$section]['national_code'], '');
			$this->_saveKeyVal("international_code", $misc[$section]['international_code'], '');
			$this->_saveKeyVal("show_app", $misc[$section]['sessions-disable'], '');
			$this->_saveKeyVal("set_hw", $misc[$section]['set_hw'], '');
			$this->_saveKeyVal("lang", $misc[$section]['lang'], '');
			$this->_saveKeyVal("gui_mode", $misc[$section]['gui_mode'], 'simple');


			//API Password
			$apipass = $misc[$section]['api-pass'];
			if (strlen($apipass) != 0) {
				require_once('/usr/local/php/include/apiusers.Class.php');
				$apiusers = new apiusers($apipass);
				$apiusers->createFile();
				$this->log("MISC.CONF:api-pass=" . $apipass . "\n");
			}

			//Cloud Settings
			$this->_saveKeyVal("cloudAddress", $misc[$section]['cloudAddress'], '');
			$this->_rootDb("cloud_enable", $misc[$section]['cloud_enable']);
			$this->_rootDb("cloud_key", $misc[$section]['cloud_key']);
			$this->_saveKeyVal("proxyEnabled", $misc[$section]['proxyEnabled'], '');
			$this->_saveKeyVal("proxyAddress", $misc[$section]['proxyAddress'], '');
			$this->_saveKeyVal("proxyPort", $misc[$section]['proxyPort'], '');
			$this->_saveKeyVal("proxyType", $misc[$section]['proxyType'], '');
			$this->_saveKeyVal("proxyUser", $misc[$section]['proxyUser'], '');
			$this->_saveKeyVal("proxySecret", $misc[$section]['proxySecret'], '');

			//HTTPS
			$this->_saveKeyVal("httpsOnly", $misc[$section]['httpsOnly'], '');

			//SNMP
			$this->_rootDb("snmp-enabled", $misc[$section]['snmp-enabled']);
			$this->_rootDb("snmptraps", $misc[$section]['snmptraps']);
			$this->_rootDb("snmptraps_sink", $misc[$section]['snmptraps_sink']);
			$this->_rootDb("snmptraps_intervall", $misc[$section]['snmptraps_intervall']);
			$this->_rootDb("snmp-community-ro", $misc[$section]['snmp-community-ro']);
			$this->_rootDb("snmp-community-rw", $misc[$section]['snmp-community-rw']);
			$this->_rootDb("snmp-community-trap", $misc[$section]['snmp-community-trap']);
		}
		$res = unlink($this->_strpath . "misc.conf.tmp");
		return true;
	}

	function hardwareconf() {

		$result = $this->_makeParseableInifile($this->_strpath . "hardware.conf", $this->_strpath . "hardware.conf.tmp");

		$res = $this->_ba->delete("delete from isdn");
		$res = $this->_ba->delete("delete from analog");
		$res = $this->_ba->delete("delete from gsm");
		$res = $this->_ba->delete("delete from lte");
		$res = $this->_ba->delete("delete from cas");
		$res = $this->_ba->delete("delete from cas_channels");
		$cfg = parse_ini_file($this->_strpath . "hardware.conf.tmp", true);

		static $lifport_isdn = 0;
		static $lifport_analog = 0;
		static $lifport_gsm = 0;
		static $lifport_lte = 0;
		static $lifport_cas = 0;

		$master = array();
		foreach ($cfg as $section => $values) {
			$res = true;
			switch ($section) {

				case "lif0":
					$master[] = $cfg[$section]['master'];
					$lifnr = 0;
					$loop = 0;
					$res = $this->_liX($lifnr, $cfg, $section);
					break;
				case "lif1":
					$master[] = $cfg[$section]['master'];
					$lifnr = 1;
					$loop = 0;
					$res = $this->_liX($lifnr, $cfg, $section);
					break;
				case "lif2":
					$master[] = $cfg[$section]['master'];
					$lifnr = 2;
					$loop = 0;
					$res = $this->_liX($lifnr, $cfg, $section);
					break;
				case "pcm":
					if ($cfg[$section]['master'] == 1)
						$master['pcm'] = $cfg[$section]['master'];
					$this->log("Nothing to do for section: " . $section . "\n");
					$loop = 1;
					break;
				default:
					$this->log("Unknown section name: " . $section . "\n");
					$loop = 1;
					break;
			}

			if ($master['pcm'] == 1) {
				$this->_ba->set('master', 'pcm');
				unset($master['pcm']);
			} else {
				$all = true;
				foreach ($master as $key => $value) {
					if ($value == 1)
						$this->_ba->set('master', $key);
					elseif ($value != "")
						$all = false;
				}
				if ($all)
					$this->_ba->set('master', 'all');
			}

			if (!$res)
				return $this->error("hardwareconf: hardware insert failed", __LINE__);


			if ($loop)
				continue;
		}//end first for loop

		@unlink($this->strpath . "hardware.conf.tmp");
		return true;
	}

	private function _liX($lifnr, $cfg, $section) {
		$type = $cfg[$section]['type'];
		if (!in_array($type, array("bf1t1e1", "bf2t1e1", "bf4t1e1", "bf4S0",
				"bf2S0", "bf1S0", "bf1E1", "bf2E1", "bf2S02FXS", "bf4FXS", "bf8FXS",
				"bf4FXO", "bf2FXO", "bf1FXS", "bf2GSM", "bf1GSM", "bf1LTE", "bf2LTE"))) {
			$this->log("Unkown module: " . $type . "\n");
			return true;
		}

		$ports = $this->_getports($type);

		$master = $cfg[$section]['master'];

		$ntteSW = explode(",", $cfg[$section]['ntteSW']);

		$crystal_clock = explode(",", $cfg[$section]['crystal_clock']);

		$permanentl1 = explode(",", $cfg[$section]['permanentl1']);

		$crc = explode(",", $cfg[$section]['crc']);

		//CLOCK Setting
		$clock = $cfg[$section]['clock'];
		if (in_array($type, array('bf4S0', 'bf1S0', 'bf1E1', 'bf2E1', 'bf2S0', 'bf2S02FXS'))) {
			($clock == "5") ? $this->_ba->set("sync_port_li" . $lifnr, '1') : $this->_ba->set("sync_port_li" . $lifnr, '0');
		}
		if (in_array($type, array('bf1t1e1', 'bf2t1e1', 'bf4t1e1'))) {
			($clock == "255") ? $this->_ba->set("sync_port_li" . $lifnr, '1') : $this->_ba->set("sync_port_li" . $lifnr, '0');
		}
		if (in_array($type, array('bf4S0', 'bf1S0', 'bf1E1', 'bf2E1', 'bf2S0', 'bf2S02FXS', 'bf1t1e1', 'bf2t1e1', 'bf4t1e1'))) {
			($clock === "0" || $clock == "") ? $this->_ba->set("sync_port_auto" . $lifnr, '1') : $this->_ba->set("sync_port_auto" . $lifnr, '0');

			if ($master == 0)
				$clock = "0";
			$clock = $this->_getdefaultValue("CLOCK", $ports, ($clock == "") ? "0" : $clock);
			if (!$clock)
				return $this->error("hardware.conf->Clock failed", __LINE__);
		}
		//END CLOCK Setting
		//PORTSWITCH Setting
		if (in_array($type, array('bf4S0', 'bf2S02FXS', 'bf4FXS', 'bf4FXO')))
			$this->_saveKeyVal("portswitch", $cfg[$section]['portswitch']);
		//END PORTSWITCH Setting
		//COUNTRY Setting
		if (in_array($type, array('bf4FXS', 'bf1FXS', 'bf8FXS', 'bf4FXO', 'bf2FXO')))
			$this->_saveKeyVal("country", $cfg[$section]['country']);
		//END Country setting
		//BOOSTRINGER Setting
		if (in_array($type, array('bf4FXS', 'bf1FXS', 'bf8FXS', 'bf2S02FXS')))
			$this->_saveKeyVal("boostringer", $cfg[$section]['boostringer']);
		//END Boostringer setting
		//TERMINATION Setting
		if (in_array($type, array('bf4S0', 'bf2S02FXS', 'bf2S0', 'bf1S0', 'bf1E1', 'bf2E1', 'bf1t1e1', 'bf2t1e1', 'bf4t1e1')))
			$termination = $this->_getdefaultValue("TERMINATION", $ports, explode(",", $cfg[$section]['termination']));
		//END Termination setting
		//PTP Setting
		if (in_array($type, array('bf4S0', 'bf2S02FXS', 'bf2S0', 'bf1S0', 'bf1E1', 'bf2E1')))
			$ptp = $this->_getdefaultValue("PTP", $ports, explode(",", $cfg[$section]['ptp']));
		//END PTP Setting

		$porttype = null;
		if (in_array($type, array('bf4S0', 'bf2S02FXS', 'bf2S0', 'bf1S0'))) {
			$ntteHW = explode(",", $cfg[$section]['ntteHW']);
			$porttype = "BRI";
		}

		if (in_array($type, array('bf1E1', 'bf2E1'))) {
			$ptp = $this->_getdefaultValue("PTP", $ports);
			$ntteHW = $ntteSW;
			$porttype = "PRI";
		}

		if (in_array($type, array('bf4FXS', 'bf1FXS', 'bf8FXS'))) {
			$porttype = "FXS";
		}

		if (in_array($type, array('bf4FXO', 'bf2FXO'))) {
			$this->_saveKeyVal("ohs", $cfg[$section]['ohs']);
			$this->_saveKeyVal("ohs2", $cfg[$section]['ohs2']);
			$this->_saveKeyVal("rz", $cfg[$section]['rz']);
			$this->_saveKeyVal("rt", $cfg[$section]['rt']);
			$this->_saveKeyVal("rt2", $cfg[$section]['rt2']);
			$this->_saveKeyVal("dcv", $cfg[$section]['dcv']);
			$this->_saveKeyVal("ilim", $cfg[$section]['ilim']);
			$this->_saveKeyVal("mini", $cfg[$section]['mini']);
			$this->_saveKeyVal("acim", $cfg[$section]['acim']);
			$porttype = "FXO";
		}

		if (in_array($type, array('bf1t1e1', 'bf2t1e1', 'bf4t1e1'))) {
			$protocol = explode(",", $cfg[$section]['protocol']);
			$country = explode(",", $cfg[$section]['country']);
			$sigchan = explode(",", $cfg[$section]['sigchan']);
			$porttype = explode(",", $cfg[$section]['interface_type']);
			$coding = explode(",", $cfg[$section]['coding']);
		}


		if (in_array($type, array('bf2GSM', 'bf1GSM', 'bf1LTE', 'bf2LTE'))) {
			for ($i = 0; $i < $ports; $i++) {
				if (strpos($type,"LTE")){
					$res = $this->_insert('lte', array('type', 'lif', 'port_type'), array($type, $lifnr, 'LTE'));
				} else {
					$res = $this->_insert('gsm', array('type', 'lif', 'port_type'), array($type, $lifnr, 'GSM'));
				}

				if (!$res)
					return $this->error("hardware.conf->GSM insert failed", __LINE__);
			}
		}elseif (in_array($type, array('bf1t1e1', 'bf2t1e1', 'bf4t1e1'))) {
			for ($i = 0; $i < $ports; $i++) {

				$res = $this->_insert('cas', array('type', 'lif', 'port_type', 'clock', 'protocol', 'country', 'sigchan', 'termination', 'coding'), array($type, $lifnr, $porttype[$i], $clock[$i], $protocol[$i], $country[$i], $sigchan[$i], $termination[$i], $coding[$i]));
				if (!$res)
					return $this->error("hardware.conf->CAS insert failed", __LINE__);
				if (strtolower($porttype[$i]) == "cas") {
					if ($protocol[$i] == "t1")
						$channels = 24;
					elseif ($protocol[$i] == "e1")
						$channels = 31;
					$portCas = $this->_ba->rowid();
					for ($j = 1; $j <= $channels; $j++) {
						$res = $this->_insert('cas_channels', array('port'), array($portCas));
						if ($j == 16 && $protocol[$i] == "e1")
							$channel = $this->_ba->rowid();
						if (!$res)
							return $this->error("hardware.conf->CAS insert failed", __LINE__);
					}
					if ($protocol[$i] == "e1")
						$this->_ba->delete("delete from cas_channels where channel='" . $channel . "';");
					//ISDN Eintrag bei hdlc speichern
				}elseif (strtolower($porttype[$i]) == "hdlc") {
					$res = $this->_insert('isdn', array('type', 'lif', 'port_type', 'ccs'), array($type, $lifnr, 'PRI', '1'));
					if (!$res)
						return $this->error("hardware.conf->ISDN insert failed", __LINE__);
				}
			}
		}elseif ($type == "bf2S02FXS") {
			for ($i = 0; $i < 2; $i++) {
				$keys = array('type', 'lif', 'ptp', 'ntteHW', 'ntteSW',
					'lt', 'clock', 'port_type', 'permanentl1');
				$values = array($type, $lifnr, $ptp[$i], $ntteHW[$i], $ntteSW[$i], $termination[$i],
					$clock[$i], 'BRI', $permanentl1[$i]);
				$res = $this->_insert('isdn', $keys, $values);

				if (!$res)
					return $this->error("hardware.conf->ISDN insert failed", __LINE__);
			}
			for ($i = 0; $i < 2; $i++) {

				$keys = array('type', 'lif', 'port_type');
				$values = array($type, $lifnr, 'FXS');
				$res = $this->_insert('analog', $keys, $values);

				if (!$res)
					return $this->error("hardware.conf->Analog insert failed", __LINE__);
			}
		}elseif ($type == "bf4FXS" || $type == "bf1FXS" || $type == "bf8FXS" || $type == "bf4FXO" || $type == "bf2FXO") {
			for ($i = 0; $i < $ports; $i++) {

				$keys = array('type', 'lif', 'port_type');
				$values = array($type, $lifnr, $porttype);
				$res = $this->_insert('analog', $keys, $values);

				if (!$res)
					return $this->error("hardware.conf->Analog insert failed", __LINE__);
			}
		}else {
			for ($i = 0; $i < $ports; $i++) {
				$keys = array('type', 'lif', 'ptp', 'ntteHW', 'ntteSW',
					'lt', 'clock', 'port_type', 'permanentl1', 'crystal_clock', 'crc');
				$values = array($type, $lifnr, $ptp[$i], $ntteHW[$i], $ntteSW[$i], $termination[$i],
					$clock[$i], $porttype, $permanentl1[$i], $crystal_clock[$i], $crc[$i]);
				$res = $this->_insert('isdn', $keys, $values);
				if (!$res)
					return $this->error("hardware.conf->ISDN insert failed", __LINE__);
			}
		}
		return true;
	}

	private function _getports($stype) {
		$stype = trim(strtolower($stype));
		switch ($stype) {
			case bf4s0:
			case bf4fxs:
			case bf4fxo:
			case bf4t1e1:
				return 4;
				break;
			case bf1s0:
			case bf1e1:
			case bf1gsm:
			case bf1lte:
			case bf1fxs:
			case bf1t1e1:
				return 1;
				break;
			case bf2e1:
			case bf2s0:
			case bf2fxo:
			case bf2gsm:
			case bf2lte:
			case bf2s02fxs:
			case bf2t1e1:
				return 2;
				break;
			case bf8fxs:
				return 8;
				break;
			default:
				return -1;
				break;
		}
	}

	function _getdefaultValue($sname, $iports, $val = null) {

		if ($sname == "CLOCK") {

			$arr = array('0', '1', '2', '3', '4', '5', '254', '255');
			if (!in_array($val, $arr, true))
				return $this->error("hardware.conf: clock out of range", __LINE__);
			if ($val == 5 || $val == 255) {
				for ($i = 1; $i <= $iports; $i++) {
					$tmp[$i - 1] = $val;
				}
			} else {

				for ($i = 1; $i <= $iports; $i++) {


					if ($val == $i) {

						$tmp[$i - 1] = "1";
					} else {

						$tmp[$i - 1] = "0";
					}
				}
			}
			return $tmp;
		}

		if ($sname == "TERMINATION" || $sname == "PTP") {
			if (!is_array($val) || count($val) != $iports) {
				for ($i = 0; $i < $iports; $i++)
					$val[$i] = 1;
			}
			return $val;
		}
	}

}
